enum FluidBreakpoint { xs, s, m, l, xl }

class FluidBreakpointsHelper {
  static FluidBreakpoint from(double width) {
    if (width < 576)
      return FluidBreakpoint.xs;
    else if (width < 768)
      return FluidBreakpoint.s;
    else if (width < 992)
      return FluidBreakpoint.m;
    else if (width < 1200)
      return FluidBreakpoint.l;
    else
      return FluidBreakpoint.xl;
  }

  static double maxContainerWidth(FluidBreakpoint breakpoint) {
    switch (breakpoint) {
      case FluidBreakpoint.xs:
        return 576;
      case FluidBreakpoint.s:
        return 768;
      case FluidBreakpoint.m:
        return 992;
      case FluidBreakpoint.l:
        return 1200;
      case FluidBreakpoint.xl:
        return null;
    }
    return null;
  }

  static double maxFluidWidth(FluidBreakpoint breakpoint) {
    switch (breakpoint) {
      case FluidBreakpoint.xs:
        return 540;
      case FluidBreakpoint.s:
        return 720;
      case FluidBreakpoint.m:
        return 960;
      case FluidBreakpoint.l:
        return 1140;
      case FluidBreakpoint.xl:
        return 1140;
    }
    return null;
  }

  static FluidBreakpoint smallerBreakpoint(FluidBreakpoint breakpoint) {
    switch (breakpoint) {
      case FluidBreakpoint.xs:
        return null;
      case FluidBreakpoint.s:
        return FluidBreakpoint.xs;
      case FluidBreakpoint.m:
        return FluidBreakpoint.s;
      case FluidBreakpoint.l:
        return FluidBreakpoint.m;
      case FluidBreakpoint.xl:
        return FluidBreakpoint.l;
    }
    return null;
  }

  static FluidBreakpoint biggerBreakpoint(FluidBreakpoint breakpoint) {
    switch (breakpoint) {
      case FluidBreakpoint.xs:
        return FluidBreakpoint.s;
      case FluidBreakpoint.s:
        return FluidBreakpoint.m;
      case FluidBreakpoint.m:
        return FluidBreakpoint.l;
      case FluidBreakpoint.l:
        return FluidBreakpoint.xl;
      case FluidBreakpoint.xl:
        return null;
    }
    return null;
  }

  bool get isXs => FluidBreakpoint.xs == this;
  bool get isS => FluidBreakpoint.s == this;
  bool get isM => FluidBreakpoint.m == this;
  bool get isL => FluidBreakpoint.l == this;
  bool get isXl => FluidBreakpoint.xl == this;

  static bool isLargerThan(FluidBreakpoint x, FluidBreakpoint breakpoint) =>
      breakpoint.index < x.index;
  static bool isLargerThanXs(FluidBreakpoint breakpoint) =>
      isLargerThan(breakpoint, FluidBreakpoint.xs);
  static bool isLargerThanS(FluidBreakpoint breakpoint) =>
      isLargerThan(breakpoint, FluidBreakpoint.s);
  static bool isLargerThanM(FluidBreakpoint breakpoint) =>
      isLargerThan(breakpoint, FluidBreakpoint.m);
  static bool isLargerThanL(FluidBreakpoint breakpoint) =>
      isLargerThan(breakpoint, FluidBreakpoint.l);
  static bool isLargerThanXl(FluidBreakpoint breakpoint) =>
      isLargerThan(breakpoint, FluidBreakpoint.xl);

  static bool isSmallerThan(FluidBreakpoint x, FluidBreakpoint breakpoint) =>
      breakpoint.index > x.index;
  static bool isSmallerThanXs(FluidBreakpoint breakpoint) =>
      isSmallerThan(breakpoint, FluidBreakpoint.xs);
  static bool isSmallerThanS(FluidBreakpoint breakpoint) =>
      isSmallerThan(breakpoint, FluidBreakpoint.s);
  static bool isSmallerThanM(FluidBreakpoint breakpoint) =>
      isSmallerThan(breakpoint, FluidBreakpoint.m);
  static bool isSmallerThanL(FluidBreakpoint breakpoint) =>
      isSmallerThan(breakpoint, FluidBreakpoint.l);
  static bool isSmallerThanXl(FluidBreakpoint breakpoint) =>
      isSmallerThan(breakpoint, FluidBreakpoint.xl);
}
